package work.xuye.common.alert;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.annotation.AfterThrowing;
import org.aspectj.lang.annotation.Aspect;
import org.springframework.stereotype.Component;
import work.xuye.common.store.MessageMD5Store;

/**
 * @author xuye
 * @since 2023/3/6 11:10
 **/
@Slf4j
@Aspect
@Component
@RequiredArgsConstructor
public class ExceptionAspect {

    private final MessageMD5Store messageMD5Store;
    
    private final AlertProperties alertProperties;


    @AfterThrowing(pointcut = "execution(* work.xuye..*(..))", throwing = "ex")
    public void handleGlobalException(Exception ex) {
        String content = "### " + ex.getClass().getName() + "\r\n" +
                "```" + "\r\n" +
                ex.getMessage() + "\r\n" +
                "```" + "\r\n";

        boolean exist = messageMD5Store.exist(content);
        if (!exist) {
            DingMessage.init()
                    .to(alertProperties.getDingTalk().getToken())
                    .withPreview("标准资讯接入程序发生异常")
                    .withContent(content)
                    .send();
            messageMD5Store.save(content);
        } else {
            log.info("短期内已经发送过相同的异常信息，不再发送: [{}]", content);
        }
    }
}
