package work.xuye.scheduler.controller;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.cloud.stream.function.StreamBridge;
import org.springframework.messaging.Message;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RestController;
import work.xuye.common.constant.BindingConstants;
import work.xuye.common.constant.MessageConstants;
import work.xuye.common.constant.StageConstants;
import work.xuye.common.db.service.TaskManager;
import work.xuye.common.dto.HttpRequestParams;
import work.xuye.common.dto.TaskVO;
import work.xuye.common.utils.ID;

/**
 * @author xuye
 * @since 2023/2/17 23:01
 **/
@Slf4j
@RestController
@RequestMapping("/task")
@RequiredArgsConstructor
public class ScheduleController {

    private final StreamBridge streamBridge;

    private final TaskManager taskManager;

    @RequestMapping(value = "/id/{id}", method = {RequestMethod.GET, RequestMethod.POST})
    public TaskVO publish(@PathVariable Integer id) {

        TaskVO taskVO = taskManager.getTaskInfoByTaskId(id);
        HttpRequestParams requestParams = taskVO.getTask().getRequestParams();

        Message<HttpRequestParams> message = MessageBuilder
                .withPayload(requestParams)
                .setHeader(MessageConstants.CURRENT_STAGE, StageConstants.SCHEDULER)
                .setHeader(MessageConstants.TASK_TRACE_ID, ID.generate())
                .setHeader(MessageConstants.TASK_ID, taskVO.getTask().getId())
                .build();
        streamBridge.send(BindingConstants.SCHEDULER_API_OUT, message);
        log.info("@ 下发任务, ID: [{}], " + MessageConstants.TASK_TRACE_ID + ": [{}]" + ", processMode: [{}]",
                taskVO.getTask().getId(),
                message.getHeaders().get(MessageConstants.TASK_TRACE_ID),
                taskVO.getTask().getProcessMode());

        return taskVO;
    }

}
