package work.xuye.sink.spel;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import work.xuye.common.constant.CustomerDateTimeFormatter;
import work.xuye.sink.expr.JsonPathHandler;

import java.time.ZonedDateTime;
import java.time.format.DateTimeFormatter;

/**
 * @author xuye
 * @since 2023/3/8 12:36
 **/
@Slf4j
@Component
@RequiredArgsConstructor
public class CustomFunction {
    private final JsonPathHandler jsonPathHandler;

    public String jsonExtra(String json, String path) {
        String result = null;
        try {
            result = jsonPathHandler.extract(json, path);
        } catch (Exception e) {
            log.error("jsonExtra handler error, json: [{}], path: [{}]", json, path);
            throw new RuntimeException(e);
        }
        log.debug("jsonExtra handler, json: [{}], path: [{}], result: [{}]", json, path, result);
        return result;
    }


    public String rfc1123(String json, String path) {
        String before = this.jsonExtra(json, path);
        String result = null;
        try {
            result = ZonedDateTime
                    .parse(before, DateTimeFormatter.RFC_1123_DATE_TIME)
                    .toLocalDateTime()
                    .format(CustomerDateTimeFormatter.READABLE_FORMATTER);
        } catch (Exception e) {
            log.error("rfc1123 handler error, before: [{}]", before);
            throw new RuntimeException(e);
        }
        log.debug("rfc1123 handler, before: [{}], after: [{}]", before, result);
        return result;
    }
}
