package work.xuye.sink.spel;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.expression.Expression;
import org.springframework.expression.ExpressionParser;
import org.springframework.expression.spel.standard.SpelExpressionParser;
import org.springframework.expression.spel.support.StandardEvaluationContext;
import org.springframework.stereotype.Service;
import work.xuye.common.db.entity.vo.MessageMapping;

import java.util.HashMap;
import java.util.List;

/**
 * @author xuye
 * @since 2023/3/4 18:30
 **/
@Slf4j
@Service
@RequiredArgsConstructor
public class SpringExpressionLanguageEvaluator {


    public final CustomFunction cf;

    public HashMap<String, Object> evaluate(MessageMapping messageMapping, String payload) {
        HashMap<String, Object> result = new HashMap<>(32);
        List<MessageMapping.FieldMapping> fields = messageMapping.getFields();
        for (MessageMapping.FieldMapping field : fields) {
            ExpressionParser parser = new SpelExpressionParser();
            StandardEvaluationContext context = new StandardEvaluationContext();
            context.setVariable("item", payload);

            Expression expression = parser.parseExpression(field.getExpression());
            Object value = null;
            try {
                value = expression.getValue(context, this);
            } catch (Exception e) {
                throw new RuntimeException("解析表达式出错，字段名：" + field.getFieldName() +
                        "，表达式：" + field.getExpression() + ", 错误消息：" + e.getMessage());
            }
            result.put(field.getFieldName(), value);
        }
        return result;
    }


}
