package work.xuye.sink.template;

import lombok.AllArgsConstructor;
import lombok.Data;
import lombok.NoArgsConstructor;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

/**
 * @author xuye
 * @since 2023/2/18 17:17
 **/
@Data
@NoArgsConstructor
@AllArgsConstructor
public class SelectTemplate {


    private List<String> fields = new ArrayList<>();

    private String tableName;

    private List<String> conditions = new ArrayList<>();

    private List<String> orders = new ArrayList<>();

    private Integer limit;

    public static SelectTemplate getInstance() {
        return new SelectTemplate();
    }

    public SelectTemplate selectAll() {
        this.fields.add("*");
        return this;
    }

    public SelectTemplate select(String... fields) {
        this.fields.addAll(Arrays.asList(fields));
        return this;
    }

    public SelectTemplate select(List<String> fields) {
        this.fields.addAll(fields);
        return this;
    }

    public SelectTemplate from(String tableName) {
        this.tableName = tableName;
        return this;
    }

    public SelectTemplate where(String expression) {
        this.conditions.add(expression);
        return this;
    }

    public SelectTemplate where(String field, Object value) {
        this.conditions.add(field + " = " + value);
        return this;
    }

    public SelectTemplate where(String field, String operator, Object value) {
        this.conditions.add(field + " " + operator + " " + value);
        return this;
    }

    public SelectTemplate orderBy(String field) {
        this.orders.add(field + " " + "asc");
        return this;
    }

    public SelectTemplate orderBy(String field, String order) {
        this.orders.add(field + " " + order);
        return this;
    }

    public SelectTemplate limit(Integer limit) {
        this.limit = limit;
        return this;
    }

    @Override
    public String toString() {
        StringBuilder sql = new StringBuilder();
        sql.append("select ");
        sql.append(String.join(",", fields));
        sql.append(" from ");
        sql.append(tableName);
        if (conditions.size() > 0) {
            sql.append(" where ");
            sql.append(String.join(" and ", conditions));
        }
        if (orders.size() > 0) {
            sql.append(" order by ");
            sql.append(String.join(",", orders));
        }
        if (limit != null) {
            sql.append(" limit ");
            sql.append(limit);
        }
        return sql.toString();
    }


}
