package work.xuye.source.handler;

import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.messaging.Message;
import org.springframework.messaging.support.MessageBuilder;
import org.springframework.stereotype.Component;
import org.springframework.util.DigestUtils;
import work.xuye.common.constant.MessageConstants;
import work.xuye.common.constant.StageConstants;
import work.xuye.common.dto.HttpRequestParams;
import work.xuye.common.dto.HttpRes;
import work.xuye.common.enums.ResourceStatus;
import work.xuye.common.service.UrlMD5Service;
import work.xuye.common.utils.ID;
import work.xuye.source.request.RequestClient;

/**
 * @author xuye
 * @since 2023/2/19 16:37
 **/
@Slf4j
@Component
@RequiredArgsConstructor
public class SourceApiHandler {

    private final RequestClient requestClient;
    private final UrlMD5Service urlMD5Service;


    public Message<HttpRes> handle(Message<HttpRequestParams> message) {
        log.info("@@ 正在处理: [{}], " + MessageConstants.TASK_TRACE_ID + ": [{}]",
                message.getPayload().getUrl(), message.getHeaders().get(MessageConstants.TASK_TRACE_ID));
        HttpRequestParams requestParams = message.getPayload();

        HttpRes httpRes = requestClient.execute(requestParams);
        String body = httpRes.getBody();
        String md5Digest = DigestUtils.md5DigestAsHex(body.getBytes());

        ResourceStatus resourceStatus = urlMD5Service.getResourceStatus(requestParams.getUrl(), md5Digest);
        urlMD5Service.updateUrlMd5(requestParams.getUrl(), md5Digest);

        httpRes.setResourceStatus(resourceStatus);
        String traceId = ID.generate();
        log.info(" 下发SourceApiHandler处理结果，" + MessageConstants.SOURCE_TRACE_ID + ": [{}]", traceId);
        return MessageBuilder
                .withPayload(httpRes)
                .setHeader(MessageConstants.CURRENT_STAGE, StageConstants.SOURCE)
                .setHeader(MessageConstants.TASK_TRACE_ID, message.getHeaders().get(MessageConstants.TASK_TRACE_ID))
                .setHeader(MessageConstants.SOURCE_TRACE_ID, traceId)
                .setHeader(MessageConstants.TASK_ID, message.getHeaders().get(MessageConstants.TASK_ID))
                .setHeader(MessageConstants.SEED_URL, requestParams.getUrl())
                .build();
    }


}



