package work.xuye.source.request;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;
import work.xuye.common.dto.HttpRequestParams;
import work.xuye.common.dto.HttpRes;

import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.http.HttpClient;
import java.net.http.HttpRequest;
import java.net.http.HttpResponse;

/**
 * @author xuye
 * @since 2023/2/18 00:22
 **/
@Slf4j
@Component
public class JdkRequestClient implements RequestClient {

    @Override
    public HttpRes execute(HttpRequestParams request) {
        HttpRequest httpRequest = null;
        try {
            httpRequest = HttpRequest.newBuilder()
                    .uri(new URI(request.getUrl()))
                    .method(request.getMethod(), HttpRequest.BodyPublishers.ofString(request.getUrl()))
                    .build();
        } catch (URISyntaxException e) {
            throw new RuntimeException(e);
        }
        HttpClient httpClient = HttpClient
                .newBuilder()
                .followRedirects(HttpClient.Redirect.ALWAYS)
                .build();

        HttpResponse<String> httpResponse = null;
        try {
            httpResponse = httpClient.send(httpRequest, HttpResponse.BodyHandlers.ofString());
        } catch (IOException | InterruptedException e) {
            throw new RuntimeException(e);
        }
        if (httpResponse.statusCode() != 200) {
            log.warn("http code is not 200, code is {}", httpResponse.statusCode());
            log.warn("http body is {}", httpResponse.body());
        }
        return HttpRes.build()
                .url(request.getUrl())
                .body(httpResponse.body())
                .status(httpResponse.statusCode())
                .headers(httpResponse.headers().map());
    }
}
